#!/bin/bash

# (c) 2011, Tjark Weber <tjark.weber@gmx.de>.  All rights reserved.
# Last modified: 2011-12-09

# This script applies QBV to all .qbc (QBF certificate) files in
# subdirectories of the working directory.  Corresponding .qdimacs
# benchmark files are expected in $qdimacs_dir.  A log file is
# generated that contains QBV's output and some timing data.

# This script was tested with QBV 2.0 (compiled Oct 26 2010), the
# latest version of QBV at the time of writing.  QBV is currently
# available from http://www.cprover.org/qbv/

# Requires (relatively small amounts of) free disk space for the
# generated log file.

###############################################################################
# Preliminaries
###############################################################################

set -e  # abort this shell script if anything goes wrong
set -u  # forbid use of uninitialized variables

###############################################################################
# Configuration - change as necessary
###############################################################################

qbv="qbv"                       # path/to/qbv-executable
qdimacs_dir="preliminary2006"   # path/to/qdimacs-benchmarks
logfile="qbv.log"               # name of log file
timeout=600                     # timeout for each QBV call (in seconds)
maxmemory=4194304               # memory limit (in KB)

###############################################################################

[ -d "$qdimacs_dir" ] ||
  (echo "$qdimacs_dir does not exist (or is not a directory)."; exit 1)

[ ! -e "$logfile" ] ||
  (echo "File $logfile exists, please delete."; exit 1)

# let's save some system information
uname -a > uname-a
cat /proc/cpuinfo > cpuinfo
cat /proc/meminfo > meminfo
"$qbv" --v > qbv-version

TIMEFORMAT=$'real\t%3R\nuser\t%3U\nsys\t%3S'

# run QBV on every certificate, save the output
for cert in `find . -name '*.qbc'|sort`; do
  benchmark="$qdimacs_dir/`basename "$cert" .qbc`"
  [ -f "$benchmark" ] ||
    (echo "File $benchmark does not exist (or is not a regular file)."; exit 1)
  echo "$cert" >> "$logfile"
  exitstatus=0
  (ulimit -t "$timeout" -v "$maxmemory" && time "$qbv" "$benchmark" "$cert") &>> "$logfile" || exitstatus=$?
  echo "exit-status $exitstatus" >> "$logfile"
done
